@@ -88,12 +88,6 @@ pub struct Endpoint<Cfg: Config> {
     max_mtu: MaxMtu,
 }
 
-/// Safety: The endpoint is marked as `!Send`, because the struct contains `Rc`s.
-/// However those `Rcs` are only referenced by other objects within the `Endpoint`
-/// and which also get moved.
-#[allow(unknown_lints, clippy::non_send_fields_in_send_ty)]
-unsafe impl<Cfg: Config> Send for Endpoint<Cfg> {}
-
 impl<Cfg: Config> s2n_quic_core::endpoint::Endpoint for Endpoint<Cfg> {
     type PathHandle = Cfg::PathHandle;
     type Subscriber = Cfg::EventSubscriber;
